package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

func InitForNiuBeiRelease(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

const PessimismLockKeyForNiuBeiRelease = "daily_settlement_niu_bei_release_pessimism_lock_key"
const PessimismLockValueForNiuBeiRelease = "running"
const DealUserCoinForNiuBeiReleaseRequestIdPrefix = "%s:integral_release_deal_user_coin:%d:uid:%d"

// DailySettlementNiuBeiRelease 每日结算“牛贝牛贝积分释放”
func DailySettlementNiuBeiRelease(engine *xorm.Engine, mid string, isTask bool) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if isTask && (now.Hour() > 8) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}

	//1、查找 `integral_release` 基础设置
	niuBeiRelease, err := db.NiuBeiCoinBasicGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if niuBeiRelease.ReleaseDate == today {
		_ = session.Rollback()
		return errors.New("今日“牛贝积分释放”已结算")
	}

	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(PessimismLockKeyForNiuBeiRelease)
	if getString == PessimismLockValueForNiuBeiRelease {
		fmt.Println("-----------------", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(PessimismLockKeyForNiuBeiRelease, PessimismLockValueForNiuBeiRelease, 3600*8) //8小时

	//TODO::关闭系统
	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "1",
	}, map[string]string{})

	//2、查询所有 `niu_bei_coin_user_with_ord` 记录
	niuBeiReleaseUserWithOrds, err := db.NiuBeiCoinUserWithOrdFindByParams(session, map[string]interface{}{
		"key":   "is_released_finish",
		"value": 0,
	})
	for _, ord := range *niuBeiReleaseUserWithOrds {
		//3、计算释放
		err, resp := CalcNiuBeiReleaseNums(session, niuBeiRelease, ord.TotalConsumeCoinNum, ord.ReleasedRatio)
		if err != nil {
			_ = session.Rollback()
			return err
		}

		//4、插入 `niu_bei_coin_user_with_ord_flow` 记录
		var niuBeiCoinUserWithOrdFlow = model.NiuBeiCoinUserWithOrdFlow{
			Uid:                  ord.Uid,
			OrdId:                ord.OrdId,
			ReleaseNiubeiCoinNum: resp.ReleaseOptionCoinNum,
			ReleaseCouponCoinNum: resp.ReleaseCouponCoinNum,
			ReleaseOptionCoinNum: resp.ReleaseOptionCoinNum,
			ReleaseDate:          today,
			CreateAt:             now,
			UpdateAt:             now,
		}
		_, err = db.NiuBeiCoinUserWithOrdFlowInsert(session, &niuBeiCoinUserWithOrdFlow)
		if err != nil {
			_ = session.Rollback()
			return err
		}

		//5、处理积分释放
		err = ReleaseNiuBeiCoinForUser(session, mid, niuBeiRelease, &ord, resp)
		if err != nil {
			_ = session.Rollback()
			return err
		}
	}

	//6、更新 `niu_bei_coin_basic` 表
	niuBeiRelease.ReleaseDate = today
	updateAffected, err := db.NiuBeiCoinBasicUpdate(session, niuBeiRelease.Id, niuBeiRelease)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if updateAffected == 0 {
		_ = session.Rollback()
		return errors.New("更新 niu_bei_coin_user_with_ord 记录失败")
	}

	//7、提交事务
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}

	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "0",
	}, map[string]string{})

	//cache.Del(PessimismLockKeyForNiuBeiRelease)

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>牛贝积分释放结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
	return nil
}

type CalcNiuBeiReleaseResp struct {
	ReleaseNiuBeiCoinNum               string // 释放牛贝积分数量
	ReleaseNiuBeiDestroyConsumeCoinNum string // 释放牛贝积分-消耗消费积分数量
	ReleaseCouponCoinNum               string // 释放抵扣劵数量
	ReleaseCouponDestroyConsumeCoinNum string // 释放抵扣劵数量-消耗消费积分数量
	ReleaseOptionCoinNum               string // 释放期权积分数量
	ReleaseOptionDestroyConsumeCoinNum string // 释放期权积分数量-消耗消费积分数量
	ReleasedRatio                      string // 已释放百分比
	IsReleasedFinish                   bool   // 是否释放完毕
}

// CalcNiuBeiReleaseNums 计算释放释放数量
func CalcNiuBeiReleaseNums(session *xorm.Session, niuBeiCoinBasic *model.NiuBeiCoinBasic, totalConsumeCoinNumValue, releasedRatioValue string) (err error, resp CalcNiuBeiReleaseResp) {
	totalCoinReleaseRatio, _ := decimal.NewFromString(niuBeiCoinBasic.TotalCoinReleaseRatio)               //总积分释放比例
	dailyConsumeCoinReleaseRatio, _ := decimal.NewFromString(niuBeiCoinBasic.DailyConsumeCoinReleaseRatio) //每日消费积分释放比例
	dailyNiubeiCoinReleaseRatio, _ := decimal.NewFromString(niuBeiCoinBasic.DailyNiubeiCoinReleaseRatio)   //每日牛贝积分释放比例
	dailyCouponCoinReleaseRatio, _ := decimal.NewFromString(niuBeiCoinBasic.DailyCouponCoinReleaseRatio)   //每日抵扣劵释放比例
	totalConsumeCoinNum, _ := decimal.NewFromString(totalConsumeCoinNumValue)                              //总消费积分数量
	releasedRatio, _ := decimal.NewFromString(releasedRatioValue)                                          //已释放百分比
	percentageValue := decimal.NewFromFloat(100)
	totalCoinRelease := dailyConsumeCoinReleaseRatio.Div(percentageValue).Mul(totalConsumeCoinNum) //总释放积分

	//1、获取各积分的兑换比例
	var coinOne, coinTwo, coinThree, coinFour model.VirtualCoin
	var coinOneValue, coinTwoValue, coinThreeValue, coinFourValue decimal.Decimal
	has, err := session.Where("id = ?", niuBeiCoinBasic.Coin1).Get(&coinOne)
	if err != nil {
		return err, resp
	}
	if has == false {
		return errors.New("未查询到相应的 消费积分 记录"), resp
	}
	coinOneValue, _ = decimal.NewFromString(coinOne.ExchangeRatio)

	has, err = session.Where("id = ?", niuBeiCoinBasic.Coin1).Get(&coinTwo)
	if err != nil {
		return err, resp
	}
	if has == false {
		return errors.New("未查询到相应的 牛贝积分 记录"), resp
	}
	coinTwoValue, _ = decimal.NewFromString(coinTwo.ExchangeRatio)

	has, err = session.Where("id = ?", niuBeiCoinBasic.Coin1).Get(&coinThree)
	if err != nil {
		return err, resp
	}
	if has == false {
		return errors.New("未查询到相应的 抵扣劵 记录"), resp
	}
	coinThreeValue, _ = decimal.NewFromString(coinThree.ExchangeRatio)

	has, err = session.Where("id = ?", niuBeiCoinBasic.Coin1).Get(&coinFour)
	if err != nil {
		return err, resp
	}
	if has == false {
		return errors.New("未查询到相应的 期权 记录"), resp
	}
	coinFourValue, _ = decimal.NewFromString(coinFour.ExchangeRatio)

	//2、计算各个积分 (TODO:: 总释放积分 * 每日牛贝积分释放比例 * （消费积分兑换比例 / 牛贝积分兑换比例）)
	resp.ReleaseNiuBeiCoinNum = totalCoinRelease.Mul(dailyNiubeiCoinReleaseRatio.Mul(percentageValue)).Mul(coinOneValue.Div(coinTwoValue)).String()
	resp.ReleaseNiuBeiDestroyConsumeCoinNum = totalCoinRelease.Mul(dailyNiubeiCoinReleaseRatio.Mul(percentageValue)).Mul(coinOneValue).String()

	resp.ReleaseCouponCoinNum = totalCoinRelease.Mul(dailyCouponCoinReleaseRatio.Mul(percentageValue)).Mul(coinOneValue.Div(coinThreeValue)).String()
	resp.ReleaseCouponDestroyConsumeCoinNum = totalCoinRelease.Mul(dailyCouponCoinReleaseRatio.Mul(percentageValue)).Mul(coinOneValue).String()
	resp.ReleasedRatio = releasedRatio.Add(dailyConsumeCoinReleaseRatio).String()
	resp.IsReleasedFinish = releasedRatio.Add(dailyConsumeCoinReleaseRatio).GreaterThanOrEqual(totalCoinReleaseRatio)
	if resp.IsReleasedFinish {
		//TODO:: （总消费积分数量 * (100 - 已释放百分比) / 100） *（消费积分兑换比例 / 牛贝积分兑换比例）
		resp.ReleaseOptionCoinNum = totalConsumeCoinNum.Mul(decimal.NewFromFloat(100).Sub(releasedRatio.Add(dailyConsumeCoinReleaseRatio)).Div(percentageValue)).Mul(coinOneValue.Div(coinFourValue)).String()
		resp.ReleaseOptionDestroyConsumeCoinNum = totalConsumeCoinNum.Mul(decimal.NewFromFloat(100).Sub(releasedRatio.Add(dailyConsumeCoinReleaseRatio)).Div(percentageValue)).Mul(coinOneValue).String()
	}
	return nil, resp
}

// ReleaseNiuBeiCoinForUser 释放用户积分
func ReleaseNiuBeiCoinForUser(session *xorm.Session, mid string, niuBeiRelease *model.NiuBeiCoinBasic, ord *model.NiuBeiCoinUserWithOrd, resp CalcNiuBeiReleaseResp) (err error) {
	beforeReleasedNiubeiCoinNum, _ := decimal.NewFromString(ord.ReleasedNiubeiCoinNum)
	releasedNiubeiCoinNum, _ := decimal.NewFromString(resp.ReleaseNiuBeiCoinNum)
	beforeReleasedCouponCoinNum, _ := decimal.NewFromString(ord.ReleasedCouponCoinNum)
	releaseCouponCoinNum, _ := decimal.NewFromString(resp.ReleaseCouponCoinNum)
	beforeReleasedOptionCoinNum, _ := decimal.NewFromString(ord.ReleasedOptionCoinNum)
	releaseOptionCoinNum, _ := decimal.NewFromString(resp.ReleaseOptionCoinNum)

	//1、牛贝积分-释放牛贝积分
	err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
		Kind:         "add",
		Mid:          mid,
		Title:        md.NiuBeiCoinByReleaseNiuBeiCoinTitleForUserVirtualCoinFlow,
		TransferType: md.NiuBeiCoinByReleaseNiuBeiCoinTransferTypeForUserVirtualCoinFlow,
		OrdId:        ord.OrdId,
		CoinId:       niuBeiRelease.Coin2,
		Uid:          ord.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseNiuBeiCoinNum),
	})
	if err != nil {
		return err
	}

	//2、释放牛贝积分-消耗消费积分数量
	err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          mid,
		Title:        md.NiuBeiCoinByReleaseNiuBeiDestroyConsumeCoinNumTitleForUserVirtualCoinFlow,
		TransferType: md.NiuBeiCoinByReleaseNiuBeiDestroyConsumeCoinNumTransferTypeForUserVirtualCoinFlow,
		OrdId:        ord.OrdId,
		CoinId:       niuBeiRelease.Coin1,
		Uid:          ord.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseNiuBeiDestroyConsumeCoinNum),
	})
	if err != nil {
		return err
	}

	//3、牛贝积分-释放抵扣劵
	err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
		Kind:         "add",
		Mid:          mid,
		Title:        md.NiuBeiCoinByReleaseCouponCoinNumTitleForUserVirtualCoinFlow,
		TransferType: md.NiuBeiCoinByReleaseCouponCoinNumTransferTypeForUserVirtualCoinFlow,
		OrdId:        ord.OrdId,
		CoinId:       niuBeiRelease.Coin2,
		Uid:          ord.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseCouponCoinNum),
	})
	if err != nil {
		return err
	}
	//4、释放抵扣劵数量-消耗消费积分数量
	err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          mid,
		Title:        md.NiuBeiCoinByReleaseCouponDestroyConsumeCoinNumTitleForUserVirtualCoinFlow,
		TransferType: md.NiuBeiCoinByReleaseCouponDestroyConsumeCoinNumTransferTypeForUserVirtualCoinFlow,
		OrdId:        ord.OrdId,
		CoinId:       niuBeiRelease.Coin2,
		Uid:          ord.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseCouponDestroyConsumeCoinNum),
	})
	if err != nil {
		return err
	}

	if resp.IsReleasedFinish {
		//5、牛贝积分-释放期权积分
		err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          mid,
			Title:        md.NiuBeiCoinByReleaseOptionCoinNumTitleForUserVirtualCoinFlow,
			TransferType: md.NiuBeiCoinByReleaseOptionCoinNumTransferTypeForUserVirtualCoinFlow,
			OrdId:        ord.OrdId,
			CoinId:       niuBeiRelease.Coin2,
			Uid:          ord.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseOptionCoinNum),
		})
		if err != nil {
			return err
		}

		//6、释放期权积分数量-消耗消费积分数量
		err = DealUserCoinForNiuBeiRelease(session, md.DealUserCoinReq{
			Kind:         "sub",
			Mid:          mid,
			Title:        md.NiuBeiCoinByReleaseOptionDestroyConsumeCoinNumTitleForUserVirtualCoinFlow,
			TransferType: md.NiuBeiCoinByReleaseOptionDestroyConsumeCoinNumTransferTypeForUserVirtualCoinFlow,
			OrdId:        ord.OrdId,
			CoinId:       niuBeiRelease.Coin2,
			Uid:          ord.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(resp.ReleaseOptionDestroyConsumeCoinNum),
		})
		if err != nil {
			return err
		}
		ord.ReleasedOptionCoinNum = beforeReleasedOptionCoinNum.Add(releaseOptionCoinNum).String()
	}
	//8、修改 `niu_bei_coin_user_with_ord` 记录
	ord.ReleasedNiubeiCoinNum = beforeReleasedNiubeiCoinNum.Add(releasedNiubeiCoinNum).String()
	ord.ReleasedCouponCoinNum = beforeReleasedCouponCoinNum.Add(releaseCouponCoinNum).String()
	ord.ReleasedRatio = resp.ReleasedRatio
	if resp.IsReleasedFinish {
		ord.IsReleasedFinish = 1 //释放完毕
	}
	updateAffected, err := db.NiuBeiCoinUserWithOrdUpdate(session, ord.Id, ord)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		err = errors.New("更新 niu_bei_coin_user_with_ord 记录失败")
		return err
	}
	return nil
}

// DealUserCoinForNiuBeiRelease 处理给用户虚拟币积分
func DealUserCoinForNiuBeiRelease(session *xorm.Session, req md.DealUserCoinReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(DealUserCoinForIntegralReleaseRequestIdPrefix, req.Mid, req.CoinId, req.Uid)
	cb, err := svc.HandleDistributedLock(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	coinAmount, err := svc.GetUserCoinAmount(session, req.Mid, req.CoinId, req.Uid)
	if err != nil {
		return err
	}
	coinAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(coinAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(4)

	var userVirtualCoinFlow model.UserVirtualCoinFlow
	userVirtualCoinFlow.CoinId = req.CoinId
	userVirtualCoinFlow.Title = req.Title
	userVirtualCoinFlow.TransferType = req.TransferType
	userVirtualCoinFlow.Uid = req.Uid
	userVirtualCoinFlow.ToUid = req.ToUid
	userVirtualCoinFlow.OrdId = req.OrdId
	userVirtualCoinFlow.BeforeAmout = coinAmount
	userVirtualCoinFlow.Amout = amountValue.String()
	userVirtualCoinFlow.CreateTime = now
	if req.Kind == "add" {
		userVirtualCoinFlow.Direction = 1
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Add(amountValue).RoundFloor(4).String()
	} else if req.Kind == "sub" {
		userVirtualCoinFlow.Direction = 2
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Sub(amountValue).RoundFloor(4).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(userVirtualCoinFlow.AfterAmout) < 0 {
		var coin model.VirtualCoin
		_, err = session.Where("id = ?", req.CoinId).Get(&coin)
		if err != nil {
			return err
		}
		zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           userVirtualCoinFlow.Uid,
			"amount":        userVirtualCoinFlow.Amout,
			"before_amount": userVirtualCoinFlow.BeforeAmout,
			"after_amount":  userVirtualCoinFlow.AfterAmout,
			"coin_id":       userVirtualCoinFlow.CoinId,
		}))
		return errors.New("用户" + zhios_order_relate_utils.IntToStr(userVirtualCoinFlow.Uid) + "的" + coin.Name + "不足")
		//userVirtualCoinFlow.AfterAmout = "0"
	}
	//3、插入 `user_virtual_coin_flow` 记录
	_, err = db.UserVirtualCoinFlowInsert(session, &userVirtualCoinFlow)
	if err != nil {
		return err
	}

	//4、修改 `user_virtual_amount`的amount值 && 及缓存
	err = svc.SetCacheUserVirtualAmount(session, req.Mid, userVirtualCoinFlow.AfterAmout, req.CoinId, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}
